@@ -1,12 +1,11 @@
-import yaml
 import logging
 
-from dvc.repo import locked
-from dvc.path_info import PathInfo
-from dvc.compat import fspath_py35
-from dvc.exceptions import DvcException
-from dvc.dependency.param import ParamsDependency
+import yaml
 
+from dvc.dependency.param import ParamsDependency
+from dvc.exceptions import DvcException
+from dvc.path_info import PathInfo
+from dvc.repo import locked
 
 logger = logging.getLogger(__name__)
 
@@ -30,10 +29,10 @@ def _collect_configs(repo):
 def _read_params(repo, configs, rev):
     res = {}
     for config in configs:
-        if not repo.tree.exists(fspath_py35(config)):
+        if not repo.tree.exists(config):
             continue
 
-        with repo.tree.open(fspath_py35(config), "r") as fobj:
+        with repo.tree.open(config, "r") as fobj:
             try:
                 res[str(config)] = yaml.safe_load(fobj)
             except yaml.YAMLError:
@@ -59,13 +58,13 @@ def show(repo, revs=None):
     if not res:
         raise NoParamsError("no parameter configs files in this repository")
 
-    # Hide working tree params if they are the same as in the active branch
+    # Hide workspace params if they are the same as in the active branch
     try:
         active_branch = repo.scm.active_branch()
     except TypeError:
         pass  # Detached head
     else:
-        if res.get("working tree") == res.get(active_branch):
-            res.pop("working tree", None)
+        if res.get("workspace") == res.get(active_branch):
+            res.pop("workspace", None)
 
     return res
